package com.ruoyi.web.controller.system;

import com.alibaba.fastjson2.JSON;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.e.order.domain.order.EOrder;
import com.ruoyi.e.order.domain.order.EOrderQuery;
import com.ruoyi.e.order.domain.order.EOrderVO;
import com.ruoyi.e.order.service.IEOrderService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 订单Controller
 *
 * @author qzz
 * @date 2024-09-19
 */
@RestController
@RequestMapping("/system/order")
public class EOrderController extends BaseController
{
    @Autowired
    private IEOrderService orderService;

    /**
     * 查询订单列表
     */
    @PreAuthorize("@ss.hasPermi('system:order:list')")
    @GetMapping("/list")
    public TableDataInfo list(EOrderQuery rlOrder)
    {
        startPage();
        List<EOrderVO> list = orderService.selectRlOrderList(rlOrder);
        return getDataTable(list);
    }

    /**
     * 导出订单列表
     */
    @PreAuthorize("@ss.hasPermi('system:order:export')")
    @Log(title = "订单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EOrderQuery rlOrder)
    {
        List<EOrderVO> list = orderService.selectRlOrderList(rlOrder);
        ExcelUtil<EOrderVO> util = new ExcelUtil<EOrderVO>(EOrderVO.class);
        util.exportExcel(response, list, "订单数据");
    }

    /**
     * 获取订单详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:order:query')")
    @GetMapping(value = "/{orderId}")
    public AjaxResult getInfo(@PathVariable("orderId") Long orderId)
    {
        return success(orderService.selectRlOrderByOrderId(orderId));
    }

    /**
     * 新增订单
     */
    @PreAuthorize("@ss.hasPermi('system:order:add')")
    @Log(title = "订单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EOrderQuery rlOrder)
    {
        return toAjax(orderService.insertRlOrder(rlOrder));
    }

    /**
     * 修改订单
     */
    @PreAuthorize("@ss.hasPermi('system:order:edit')")
    @Log(title = "订单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EOrder rlOrder)
    {
        return toAjax(orderService.updateRlOrder(rlOrder));
    }


    /**
     * 退款
     */
    @PreAuthorize("@ss.hasPermi('system:order:refund')")
    @Log(title = "订单退款", businessType = BusinessType.REFUND)
    @PutMapping("/refund")
    public AjaxResult orderRefund(@RequestBody EOrderQuery etOrder)
    {
        logger.info("【订单退款请求】：{}", JSON.toJSON(etOrder));
        return toAjax(orderService.refund(etOrder));
    }
}
