package com.ruoyi.e.order.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.EUserVO;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.e.order.domain.order.EOrderVO;
import com.ruoyi.e.order.domain.order.bo.EOrderBO;
import com.ruoyi.e.order.domain.order.dto.EOrderDTO;
import com.ruoyi.e.order.domain.order.dto.EReletOrderDTO;
import com.ruoyi.e.order.service.IEOrderConverter;
import com.ruoyi.e.order.service.IEOrderService;
import com.ruoyi.system.domain.channel.ChannelVO;
import com.ruoyi.system.domain.device.EDevice;
import com.ruoyi.system.domain.model.EModelVO;
import com.ruoyi.system.domain.rule.EFeeRuleVO;
import com.ruoyi.system.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

@Service
public class EOrderConverterImpl implements IEOrderConverter {


    @Resource
    private IEFeeRuleService feeRuleService;

    @Resource
    private IEUserService userService;

    @Autowired
    private IEOrderService orderService;

    @Autowired
    private IEDeviceService deviceService;

    @Autowired
    private EChannelService channelService;

    @Autowired
    private IEModelService modelService;


    // 订单dto转bo
    @Override
    public EOrderBO toOrderBO(EOrderDTO order) {

        // 车辆
        EDevice eDevice = deviceService.selectEDeviceBySn(order.getSn());
        ServiceUtil.assertion(eDevice == null, "设备不存在");

        // 车型
        EModelVO model = modelService.selectEModelByModelId(eDevice.getModelId());
        ServiceUtil.assertion(model == null, "设备【"+order.getSn()+"】的车型不存在");
        ServiceUtil.assertion(eDevice.getUserId() == null, "设备【"+order.getSn()+"】的商户不存在");


        // 商户
        EUserVO merchantVO =  userService.selectUserById(eDevice.getUserId());
        ServiceUtil.assertion(ObjectUtil.isNull(merchantVO), "商户不存在");

        ServiceUtil.assertion(merchantVO.getServiceFeeProportion() == null, "商户【"+merchantVO.getUserName()+"】的平台服务费未配置");
        ServiceUtil.assertion(merchantVO.getPayChannel() == null, "商户【"+merchantVO.getUserName()+"】的支付渠道未配置");

        // 支付渠道
        ChannelVO channelVO = channelService.selectSmChannelByChannelId(merchantVO.getPayChannel());
        ServiceUtil.assertion(channelVO == null, "支付渠道不存在");

        // 套餐
        EFeeRuleVO eFeeRuleVO = feeRuleService.selectRlFeeRuleByRuleId(order.getRuleId());
        ServiceUtil.assertion(eFeeRuleVO == null, "套餐不存在");

        // 用户
        EUserVO eUserVO = userService.selectUserById(SecurityUtils.getUserId());
        ServiceUtil.assertion(eUserVO == null, "用户不存在");
        ServiceUtil.assertion(eUserVO.getWxopenid() == null, "用户微信openId为空");

        EOrderBO orderBO = new EOrderBO();
        orderBO.setParams(order);
        orderBO.setFeeRule(eFeeRuleVO);
        orderBO.setMerchant(merchantVO);
        orderBO.setChannel(channelVO);
        orderBO.setUser(eUserVO);
        orderBO.setDevice(eDevice);
        orderBO.setModel(model);
        orderBO.setType(ServiceConstants.ORDER_TYPE_LEASE);
        return orderBO;
    }

    // 订单dto转bo
    @Override
    public EOrderBO toOrderBO(EReletOrderDTO order) {

        // 原订单
        EOrderVO originalOrder = orderService.selectRlOrderByOrderNo(order.getOrderNo());
        ServiceUtil.assertion(originalOrder == null, "原订单不存在");

        // 车辆
        EDevice eDevice = deviceService.selectEDeviceBySn(originalOrder.getSn());
        ServiceUtil.assertion(eDevice == null, "设备不存在");

        // 车型
        EModelVO model = modelService.selectEModelByModelId(eDevice.getModelId());
        ServiceUtil.assertion(model == null, "设备【"+eDevice.getSn()+"】的车型不存在");


        // 商户
        EUserVO merchantVO =  userService.selectUserById(originalOrder.getMerchantId());
        ServiceUtil.assertion(merchantVO == null, "商户不存在");
        ServiceUtil.assertion(merchantVO.getServiceFeeProportion() == null, "商户【"+merchantVO.getUserName()+"】的平台服务费未配置");
        ServiceUtil.assertion(merchantVO.getPayChannel() == null, "商户【"+merchantVO.getUserName()+"】的支付渠道未配置");

        // 支付渠道
        ChannelVO channelVO = channelService.selectSmChannelByChannelId(merchantVO.getPayChannel());
        ServiceUtil.assertion(channelVO == null, "支付渠道不存在");

        // 套餐
        EFeeRuleVO eFeeRuleVO = feeRuleService.selectRlFeeRuleByRuleId(order.getRuleId());
        ServiceUtil.assertion(eFeeRuleVO == null, "套餐不存在");

        // 用户
        EUserVO eUserVO = userService.selectUserById(originalOrder.getUserId());
        ServiceUtil.assertion(eUserVO == null, "用户不存在");
        ServiceUtil.assertion(eUserVO.getWxopenid() == null, "用户微信openId为空");

        EOrderBO orderBO = new EOrderBO();
        orderBO.setFeeRule(eFeeRuleVO);
        orderBO.setReletOrderDTO(order);
        orderBO.setMerchant(merchantVO);
        orderBO.setUser(eUserVO);
        orderBO.setChannel(channelVO);
        orderBO.setOriginalOrder(originalOrder);
        orderBO.setType(ServiceConstants.ORDER_TYPE_RELET);
        orderBO.setDevice(eDevice);
        orderBO.setModel(model);
        return orderBO;
    }


}
