package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.EUserVO;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.e.order.domain.order.EOrder;
import com.ruoyi.system.domain.dividendDetail.DividendDetail;
import com.ruoyi.system.domain.dividendDetail.DividendDetailQuery;
import com.ruoyi.system.domain.dividendDetail.DividendDetailVO;
import com.ruoyi.system.mapper.RlDividendDetailMapper;
import com.ruoyi.system.service.IEChangeBalanceService;
import com.ruoyi.system.service.IEUserService;
import com.ruoyi.system.service.IDividendDetailService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

/**
 * 分账明细Service业务层处理
 *
 * @author qzz
 * @date 2024-10-10
 */
@Slf4j
@Service
public class DividendDetailServiceImpl implements IDividendDetailService
{
    @Resource
    private RlDividendDetailMapper rlDividendDetailMapper;

    @Resource
    private IEUserService userService;

    @Autowired
    private IEChangeBalanceService changeBalanceService;

    /**
     * 查询分账明细
     *
     * @param id 分账明细主键
     * @return 分账明细
     */
    @Override
    public DividendDetail selectRlDividendDetailById(Long id)
    {
        return rlDividendDetailMapper.selectRlDividendDetailById(id);
    }

    /**
     * 查询分账明细列表
     *
     * @param rlDividendDetail 分账明细
     * @return 分账明细
     */
    @Override
//    @DataScope(agentAlias = "a",userAlias = "u")
    public List<DividendDetailVO> selectRlDividendDetailList(DividendDetailQuery rlDividendDetail)
    {
        return rlDividendDetailMapper.selectRlDividendDetailList(rlDividendDetail);
    }

    /**
     * 根据订单号查询分账明细列表
     * @param orderNo
     * @return
     */
    @Override
    public List<DividendDetailVO> selectRlDividendDetailListByOrderNo(String orderNo) {
        return rlDividendDetailMapper.selectRlDividendDetailListByOrderNo(orderNo);
    }

    /**
     * 新增分账明细
     *
     * @param rlDividendDetail 分账明细
     * @return 结果
     */
    @Override
    public int insertRlDividendDetail(DividendDetail rlDividendDetail)
    {
        rlDividendDetail.setCreateTime(DateUtils.getNowDate());
        return rlDividendDetailMapper.insertRlDividendDetail(rlDividendDetail);
    }

    /**
     * 修改分账明细
     *
     * @param rlDividendDetail 分账明细
     * @return 结果
     */
    @Override
    public int updateRlDividendDetail(DividendDetail rlDividendDetail)
    {
        return rlDividendDetailMapper.updateRlDividendDetail(rlDividendDetail);
    }

    /**
     * 批量删除分账明细
     *
     * @param ids 需要删除的分账明细主键
     * @return 结果
     */
    @Override
    public int deleteRlDividendDetailByIds(Long[] ids)
    {
        return rlDividendDetailMapper.deleteRlDividendDetailByIds(ids);
    }

    /**
     * 删除分账明细信息
     *
     * @param id 分账明细主键
     * @return 结果
     */
    @Override
    public int deleteRlDividendDetailById(Long id)
    {
        return rlDividendDetailMapper.deleteRlDividendDetailById(id);
    }

    /**
     * 计算分账金额
     * @param originalOrder 原始订单对象
     * @return
     */
    @Override
    public int calculationDividend(EOrder originalOrder) {
        EUserVO user = userService.selectUserById(originalOrder.getMerchantId());//商户
        // todo 根据商户id查询合伙人列表
        List<EUserVO> rlUser = userService.getPartnerList(user.getUserId());//合伙人
        if(ObjectUtil.isNotEmpty(rlUser)){
            for(EUserVO userVO:rlUser){
                // 根据商户查询所有合伙人
                DividendDetail rlDividendDetail = new DividendDetail();
                rlDividendDetail.setPartnerId(user.getUserId());
                rlDividendDetail.setPartnerName(user.getUserName());
                rlDividendDetail.setPartnerPhone(user.getPhonenumber());
                rlDividendDetail.setPartnerType(user.getUserType());
                rlDividendDetail.setOrderNo(originalOrder.getOrderNo());
                rlDividendDetail.setTotalAmount(originalOrder.getPayFee());
                rlDividendDetail.setCreateTime(DateUtils.getNowDate());
                rlDividendDetail.setStatus(ServiceConstants.DIVIDEND_STATUS_NOT_SETTLED);
                int i = insertRlDividendDetail(rlDividendDetail);
                ServiceUtil.assertion(i==0, "新增合伙人分账记录失败：userId="+user.getUserId());

            }
        }
        return 1;
    }

    /**
     * 计算分账金额
     * @param rlDividendDetails 分账明细
     * @return
     */
    @Override
    public int calculationAmount(List<DividendDetailVO> rlDividendDetails, BigDecimal totalDividendAmount) {
        log.info("【计算分账金额】计算分账金额开始============");
        for(DividendDetailVO dividendDetail:rlDividendDetails){
            log.info("【计算分账金额】租赁费和逾期费进行分成============{}", totalDividendAmount);
            EUserVO eUserVO = userService.selectUserById(dividendDetail.getPartnerId());

            log.info("【计算分账金额】分账合伙人============{}", eUserVO);
            BigDecimal dividendProportion = dividendDetail.getDividendProportion();

            log.info("【计算分账金额】分账合伙人：userid={}============分账比例：{}", eUserVO.getUserId(),dividendProportion);
            if(ObjectUtil.isNull(eUserVO)){
                throw new ServiceException("用户【"+dividendDetail.getPartnerName()+"】不存在");
            }
            if(ObjectUtil.isNull(dividendProportion) || dividendProportion.compareTo(BigDecimal.ZERO)==0){
                throw new ServiceException("用户【"+dividendDetail.getPartnerName()+"】分账比例不能为空");
            }
            // 保留两位小数点
            BigDecimal dividendAmount = dividendProportion.multiply(totalDividendAmount)
                    .setScale(2, RoundingMode.HALF_UP);
            dividendDetail.setDividendAmount(dividendAmount);
            dividendDetail.setStatus(ServiceConstants.DIVIDEND_STATUS_SETTLED);
            updateRlDividendDetail(dividendDetail);
        }
        return 1;
    }

    /**
     * 更新退款金额
     *
     * @param id 分成明细id
     * @param userRefundAmount 用户退款金额
     * @return 结果
     */
    @Override
    public int updateRefundAmount(Long id, BigDecimal userRefundAmount) {
        DividendDetail rlDividendDetail = new DividendDetail();
        rlDividendDetail.setRefundAmount(userRefundAmount);
        rlDividendDetail.setId(id);
        return rlDividendDetailMapper.updateRlDividendDetail(rlDividendDetail);
    }


}
