package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServletUtils;
import com.ruoyi.system.domain.EFunction;
import com.ruoyi.system.domain.device.EDevice;
import com.ruoyi.system.domain.model.EModel;
import com.ruoyi.system.domain.model.EModelQuery;
import com.ruoyi.system.domain.model.EModelVO;
import com.ruoyi.system.domain.modelRule.EModelRule;
import com.ruoyi.system.domain.rule.EFeeRule;
import com.ruoyi.system.mapper.EModelMapper;
import com.ruoyi.system.mapper.EModelRuleMapper;
import com.ruoyi.system.service.*;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.List;

/**
 * 车辆型号Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-05-13
 */
@Slf4j
@Service
public class EModelServiceImpl implements IEModelService
{
    @Resource
    private EModelMapper eModelMapper;

    @Autowired
    private IEDeviceService eDeviceService;

    @Autowired
    private IEFunctionService rlFunctionService;

    @Autowired
    private IEBrandService brandService;

    @Autowired
    private IEFeeRuleService feeRuleService;

    @Autowired
    private EModelRuleMapper eModelRuleMapper;


    /**
     * 查询车辆型号
     *
     * @param modelId 车辆型号主键
     * @return 车辆型号
     */
    @Override
    public EModelVO selectEModelByModelId(Long modelId)
    {
        EModelVO etModel = eModelMapper.selectEModelByModelId(modelId);
        if(etModel != null){
            etModel.setFeeRulesIds(feeRuleService.selectRlFeeRuleLongListByModelId(etModel.getModelId()));
        }
        return etModel;
    }

    /**
     * 查询车辆型号列表
     *
     * @param eModel 车辆型号
     * @return 车辆型号
     */
    @Override
    @DataScope(merchantAlias = "m")
    public List<EModelVO> selectEModelList(EModel eModel)
    {
        return eModelMapper.selectEModelList(eModel);
    }

    /**
     * 根据商户id查询车辆型号列表
     *
     * @param merchantId 商户id
     * @return 车辆型号集合
     */
    @Override
    public List<EModelVO> selectEModelListByMerchantId(Long merchantId) {
        EModel model = new EModel();
        model.setMerchantId(merchantId);
        return eModelMapper.selectEModelList(model);
    }

    /**
     * 根据店铺id查询车辆型号列表ids
     *
     * @param storeId 店铺id
     * @return 车辆型号
     */
    @Override
    public List<Long> selectEModelLongListByStoreId(Long storeId) {
        return eModelMapper.selectEModelLongListByStoreId(storeId);
    }

    /**
     * 根据店铺id查询车辆型号列表
     *
     * @param storeId 店铺id
     * @return 车辆型号
     */
    @Override
    public List<EModelVO> selectEModelListByStoreId(Long storeId)
    {
        List<EModelVO> models = eModelMapper.selectEModelListByStoreId(storeId);
//        for (EModelVO model:models) {
//            List<RlFunction> functionList = getFunctionListByModelId(model.getModelId());
//            model.setFunctionList(functionList);
//
//            List<RlAccessoryVO> accessories = accessoryService.selectRlAccessoryListByModelId(model.getModelId());
//            if(ObjectUtil.isNotNull(accessories)){
//                model.setAccessorys(accessories);
//            }
//        }
        return models;
    }

    /**
     * 新增车辆型号
     *
     * @param etModel 车辆型号
     * @return 结果
     */
    @SneakyThrows
    @Override
    @Transactional
    public int insertEModel(EModelQuery etModel)
    {
        etModel.setCreateTime(DateUtils.getNowDate());
        if(etModel.getBrandId() != null){
            etModel.setBrandName(brandService.selectEBrandByBrandId(etModel.getBrandId()).getName());
        }
        int i = eModelMapper.insertEModel(etModel);
        saveModelRule(etModel);
        return i;
    }

    private void saveModelRule(EModelQuery etModel) {
        if(etModel.getFeeRulesIds() != null){
            Long[] feeRulesIds = etModel.getFeeRulesIds();
            for (Long feeRuleId:feeRulesIds){
                EModelRule eModelRule = new EModelRule();
                eModelRule.setModelId(etModel.getModelId());
                eModelRule.setRuleId(feeRuleId);
                int i1 = eModelRuleMapper.insertEModelRule(eModelRule);
            }
        }
    }

    /**
     * 修改车辆型号
     *
     * @param etModel 车辆型号
     * @return 结果
     */
    @SneakyThrows
    @Override
    @Transactional
    public int updateEModel(EModelQuery etModel)
    {
        etModel.setUpdateTime(DateUtils.getNowDate());
        if(etModel.getBrandId() != null){
            etModel.setBrandName(brandService.selectEBrandByBrandId(etModel.getBrandId()).getName());
        }
        int i = eModelRuleMapper.deleteEModelRuleByModelId(etModel.getModelId());
        saveModelRule(etModel);
        return eModelMapper.updateEModel(etModel);
    }

    /**
     * 批量删除车辆型号
     *
     * @param modelIds 需要删除的车辆型号主键
     * @return 结果
     */
    @Override
    public int deleteEModelByModelIds(Long[] modelIds)
    {
        return eModelMapper.deleteEModelByModelIds(modelIds);
    }

    /**
     * 删除车辆型号信息
     *
     * @param modelId 车辆型号主键
     * @return 结果
     */
    @Override
    public int deleteEModelByModelId(Long modelId)
    {
        return eModelMapper.deleteEModelByModelId(modelId);
    }

    @Override
    public int selectAllCount() {
        return eModelMapper.selectAllCount();
    }

    /**
     * 根据车型id获取功能
     */
    @Override
    public List<EFunction> getFunctionListByModelId(Long modelId) {
        return rlFunctionService.selectRlFunctionByModelId(modelId);
    }


}
